@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/singleton-trait/0.2.0")]
+#![doc(html_root_url = "https://docs.rs/singleton-trait/0.3.0")]
 #![no_std]
 
 use core::cell::{Cell, UnsafeCell};
@@ -65,23 +65,31 @@ unsafe impl<'a, T: Singleton> Singleton for Erased<T> {}
  * The Erased struct witnesses the logical ownership of a value of type T
  * while remaining zero-sized. This can be used for ghost proofs of soundness.
  *
- * Erased<T> should be thought of a zero-sized owner of T.
+ * Erased<T> should be thought of a zero-sized owner of T. It is useful for inclusion
+ * in data structures where the value might not be eliminated by the compiler.
+ * It likely is not necessary for short-lived data.
  *
  * NOTE: drop implementations will never be called, as Exists<T> guarantees the existence
  * of a valid T, which might not be true if they were called. On the other hand, since
  * it does not hold a T, it cannot drop T when it is itself dropped
  *
  * Secondly, keep in mind that while Erased<T> serves as evidence, it does not include
- * sufficient provenance for Stacked Borrows or LLVM, and so it is not usually sound
+ * sufficient provenance for Stacked Borrows or LLVM, and so it is not techinically sound
  * to recover a reference `&T` from an `Exists<&T>` and `*mut T` or `&UnsafeCell<T>`
  * even when T is Singleton, but this could be possible if T is zero-sized.
  * Because of the missing provenance, creating a reference this way could invalidate
  * the original reference on which this Exists instance was based.
  */
-#[derive(Clone, Copy)]
+#[derive(Copy)]
 pub struct Erased<T> {
     _phantom: PhantomData<ManuallyDrop<T>>,
 }
+impl<T: Copy> Clone for Erased<T> {
+    #[inline(always)]
+    fn clone(&self) -> Self {
+        *self
+    }
+}
 impl<T> Erased<T> {
     #[inline(always)]
     pub const fn new(t: T) -> Self {
@@ -109,6 +117,19 @@ impl<T> Erased<T> {
 
     /**
      * Turns a &Erased<T> into an Erased<&T>
+     *
+     * This can be especially useful when storing
+     * the Erased<T> inside another wrapper type
+     *
+     * ```
+     * # use singleton_trait::Erased;
+     * use core::cell::RefCell;
+     * struct Token;
+     * let lock = RefCell::new(Erased::new(Token));
+     *
+     * let locked = lock.borrow();
+     * let _: Erased<&Token> = locked.borrow();
+     * ```
      */
     #[inline(always)]
     pub fn borrow(&self) -> Erased<&T> {
@@ -119,6 +140,19 @@ impl<T> Erased<T> {
 
     /**
      * Turns a &mut Erased<T> into an Erased<&mut T>
+     *
+     * This can be especially useful when storing
+     * the Erased<T> inside another wrapper type
+     *
+     * ```
+     * # use singleton_trait::Erased;
+     * use core::cell::RefCell;
+     * struct Token;
+     * let lock = RefCell::new(Erased::new(Token));
+     *
+     * let mut locked = lock.borrow_mut();
+     * let _: Erased<&mut Token> = locked.borrow_mut();
+     * ```
      */
     #[inline(always)]
     pub fn borrow_mut(&mut self) -> Erased<&mut T> {
@@ -195,6 +229,7 @@ impl<T> Erased<T> {
             er: Erased<T>,
         }
         impl<T> Exists<T> for Internal<T> {
+            #[inline(always)]
             fn erase(self) -> Erased<T> {
                 self.er
             }
@@ -216,6 +251,27 @@ impl<T> Erased<Erased<T>> {
     }
 }
 impl<'a, T> Erased<&'a mut T> {
+    /**
+     * Re-borrow this erased mutable borrow. Usually
+     * this is implicit in Rust, but not for general wrappers.
+     * This can be seen as a "clone" method for mutable borrows,
+     * as it allows you to retain the erased borrow after the
+     * end of the lifetime.
+     *
+     * The Exists trait can perform this conversion automatically.
+     *
+     * ```
+     * # use singleton_trait::Erased;
+     *
+     * ```
+     * # use singleton_trait::Erased;
+     * struct Token;
+     * fn recursively(mut b: Erased<&mut Token>) {
+     *  recursively(b.reborrow());
+     *  recursively(b);
+     * }
+     * ```
+     */
     #[inline(always)]
     pub fn reborrow<'b>(&'b mut self) -> Erased<&'b mut T> {
         // SAFETY
@@ -223,8 +279,63 @@ impl<'a, T> Erased<&'a mut T> {
         // Refs and derefs on reference types are pure
         unsafe { self.map_borrow_mut(|r: &'b mut &'a mut T| &mut **r) }
     }
+
+    /**
+     * Borrow this erased mutable borrow as immutable.
+     * This is effectively a Deref method under Erased
+     *
+     * The Exists trait can perform this conversion automatically.
+     *
+     * ```
+     * # use singleton_trait::Erased;
+     * struct Lock;
+     * fn use_lock(write: Erased<&mut Lock>) {
+     *   read_from(write.read());
+     * }
+     *
+     * fn read_from(read: Erased<&Lock>) { }
+     * ```
+     */
+    #[inline(always)]
+    pub fn read<'b>(&'b self) -> Erased<&'b T> {
+        // SAFETY
+        //
+        // Refs and derefs on reference types are pure
+        unsafe { self.map_borrow(|r: &'b &'a mut T| &**r) }
+    }
+
+    /**
+     * Consume this erased mutable borrow to
+     * turn it into an immutable borrow.
+     *
+     * The Exists trait can perform this conversion automatically.
+     *
+     * See also `read`, which is often more useful.
+     * This method uses the full inner lifetime
+     * without needing to borrow.
+     * ```
+     * # use singleton_trait::Erased;
+     * struct Lock;
+     * struct MutWrapper<'a> (Erased<&'a mut Lock>);
+     * struct Wrapper<'a> (Erased<&'a Lock>);
+     * impl<'a> MutWrapper<'a> {
+     *   pub fn into_read(self) -> Wrapper<'a> {
+     *      Wrapper(self.0.into_shared())
+     *   }
+     * }
+     *
+     * ```
+     */
+    #[inline(always)]
+    pub fn into_shared(self) -> Erased<&'a T> {
+        // SAFETY
+        //
+        // Refs and derefs on reference types are pure
+        unsafe { self.map(|r: &'a mut T| &*r) }
+    }
 }
 impl<T> From<T> for Erased<T> {
+    #[inline(always)]
     fn from(t: T) -> Self {
         Self::new(t)
     }
@@ -248,15 +359,13 @@ impl<T> Exists<T> for T {
 impl<'a, 'b: 'a, T> Exists<&'a T> for Erased<&'b T> {
     #[inline(always)]
     fn erase(self) -> Erased<&'a T> {
-        // SAFETY: Deref on reference is pure
-        unsafe { self.map(|r: &'a T| &*r) }
+        self
     }
 }
 impl<'a, 'b: 'a, T> Exists<&'a T> for Erased<&'b mut T> {
     #[inline(always)]
     fn erase(self) -> Erased<&'a T> {
-        // SAFETY: Deref on reference is pure
-        unsafe { self.map(|r: &'a mut T| &*r) }
+        self.into_shared()
     }
 }
 impl<'a, 'b: 'a, T> Exists<&'a mut T> for Erased<&'b mut T> {
@@ -269,22 +378,21 @@ impl<'a, 'b: 'a, T> Exists<&'a mut T> for Erased<&'b mut T> {
 impl<'a, 'b, T> Exists<&'a T> for &'a Erased<&'b T> {
     #[inline(always)]
     fn erase(self) -> Erased<&'a T> {
-        // SAFETY: Deref on reference is pure
-        unsafe { self.map(|r: &'a T| &*r) }
+        *self
     }
 }
 impl<'a, 'b: 'a, T> Exists<&'a T> for &'a Erased<&'b mut T> {
     #[inline(always)]
     fn erase(self) -> Erased<&'a T> {
         // SAFETY: Deref on reference is pure
-        unsafe { self.map_borrow(|r: &'a &'b mut T| &**r) }
+        self.read()
     }
 }
 impl<'a, 'b: 'a, T> Exists<&'a mut T> for &'a mut Erased<&'b mut T> {
     #[inline(always)]
     fn erase(self) -> Erased<&'a mut T> {
         // SAFETY: Deref on reference is pure
-        unsafe { self.map_borrow_mut(|r: &'a mut &'b mut T| &mut **r) }
+        self.reborrow()
     }
 }
 
